@@ -1,35 +1,48 @@
 from .api import Api
 from typing import Any
 
+V_VANE_POSITION_UNKNOWN = "unknown"
+V_VANE_POSITION_1 = "1_up"
+V_VANE_POSITION_2 = "2"
+V_VANE_POSITION_3 = "3"
+V_VANE_POSITION_4 = "4_down"
+V_VANE_POSITION_SWING = "swing"
+
+SWING_LIST_DICT = [
+    V_VANE_POSITION_1,
+    V_VANE_POSITION_2,
+    V_VANE_POSITION_3,
+    V_VANE_POSITION_4,
+    V_VANE_POSITION_SWING,
+]
+
+_V_VANE_POSITION_LOOKUP = {
+    0: V_VANE_POSITION_UNKNOWN,
+    1: V_VANE_POSITION_1,
+    2: V_VANE_POSITION_2,
+    3: V_VANE_POSITION_3,
+    4: V_VANE_POSITION_4,
+    5: V_VANE_POSITION_SWING,
+}
+
 
 class SplitAC:
     def __init__(self, dsn: str, api_param: Api) -> None:
         self._dsn = dsn
         self._api = api_param  # Setting the API object
 
-        # Calling the api class _get_device_properties to get devices properties
-        self._properties = self._api._get_device_properties(self._dsn)
-
         # self.properties: For now this variable is not used but lots of device properties which are not implemented
         # this variable can be used to expose those properties and implement them.
-        self.device_name = self._properties
-        self.af_vertical_swing = self._properties
-        self.af_vertical_direction = self._properties
-        self.af_horizontal_swing = self._properties
-        self.af_horizontal_direction = self._properties
-        self.economy_mode = self._properties
-        self.fan_speed = self._properties
-        self.powerful_mode = self._properties
-        self.min_heat = self._properties  # TODO Missing device setting method
-        self.outdoor_low_noise = self._properties  # TODO Missing device setting method
-        self.operation_mode = self._properties  # type: ignore
-        self.adjust_temperature = self._properties
+        self.update_properties()
 
     # Method for getting new (refreshing) properties values
-    def refresh_properties(self):
+    def refresh_properties(self) -> None:
+        self.update_properties()
+
+    # Calling the api class _get_device_properties to get devices properties
+    def update_properties(self) -> None:
         self._properties = self._api._get_device_properties(self._dsn)
         self.device_name = self._properties
-        self.adjust_temperature = self._properties
         self.af_vertical_swing = self._properties
         self.af_vertical_direction = self._properties
         self.af_horizontal_swing = self._properties
@@ -39,7 +52,10 @@ class SplitAC:
         self.powerful_mode = self._properties
         self.min_heat = self._properties
         self.outdoor_low_noise = self._properties
-        self.operation_mode = self._properties  # type: ignore
+        self.operation_mode = self._properties
+        self.adjust_temperature = self._properties
+        self.display_temperature = self._properties
+        self.outdoor_temperature = self._properties
 
     # To Turn on the device get the last operation mode using property history method
     # Find the last not 'OFF'/'0' O.M.
@@ -76,7 +92,6 @@ class SplitAC:
     # Fan speed setting
     # Quiet Low Medium High Auto
     def changeFanSpeed(self, speed):
-        print(speed)
         if speed.upper() == "QUIET":
             self.fan_speed_quiet()
             return None
@@ -112,31 +127,49 @@ class SplitAC:
         FAN_SPEED_DICT = {0: "Quiet", 1: "Low", 2: "Medium", 3: "High", 4: "Auto"}
         return FAN_SPEED_DICT[self.fan_speed["value"]]
 
-    # Fan Swing mode
-    # 0: 'Horizontal',1: 'Down', 2: 'Unknown', 3: 'Swing'
-    def changeSwingMode(self, mode):
-        print(mode)
-        if mode.upper() == "HORIZONTAL":
-            self.af_vertical_direction = 0
-            return None
-        if mode.upper() == "DOWN":
+    # Vane vertical mode
+    def changeSwingMode(self, mode) -> None:
+        if mode.upper() == V_VANE_POSITION_1.upper():
             self.af_vertical_direction = 1
-            return None
-        if mode.upper() == "UNKNOWN":
+            self.af_vertical_swing = 0
+            self.af_horizontal_swing = 0
+        elif mode.upper() == V_VANE_POSITION_2.upper():
             self.af_vertical_direction = 2
-            return None
-        if mode.upper() == "SWING":
+            self.af_vertical_swing = 0
+            self.af_horizontal_swing = 0
+        elif mode.upper() == V_VANE_POSITION_3.upper():
             self.af_vertical_direction = 3
-            return None
+            self.af_vertical_swing = 0
+            self.af_horizontal_swing = 0
+        elif mode.upper() == V_VANE_POSITION_4.upper():
+            self.af_vertical_direction = 4
+            self.af_vertical_swing = 0
+            self.af_horizontal_swing = 0
+        elif mode.upper() == V_VANE_POSITION_SWING.upper():
+            self.af_vertical_swing = 1
+            self.af_horizontal_swing = 1
 
     def get_swing_mode_desc(self):
-        SWING_LIST_DICT = {0: "Horizontal", 1: "Down", 2: "Unknown", 3: "Swing"}
         try:
             return SWING_LIST_DICT[self.af_vertical_direction["value"]]
         except TypeError:
-            return SWING_LIST_DICT[2]
+            return SWING_LIST_DICT[0]
 
-    # Direction Settings
+    def vane_vertical_positions(self) -> list[str]:
+        """Return available vertical vane positions."""
+        return SWING_LIST_DICT
+
+    def vane_vertical(self) -> str:
+        """Return vertical vane position."""
+        verticalDirectionValue = self.af_vertical_direction["value"]
+        verticalSwingValue = self.af_vertical_swing["value"]
+        horizontalSwingValue = self.af_horizontal_swing["value"]
+        if verticalSwingValue == 1 or horizontalSwingValue == 1:
+            return V_VANE_POSITION_SWING
+        else:
+            return _V_VANE_POSITION_LOOKUP.get(
+                verticalDirectionValue, V_VANE_POSITION_UNKNOWN
+            )
 
     # Vertical
     def vertical_swing_on(self):
@@ -145,14 +178,6 @@ class SplitAC:
     def vertical_swing_off(self):
         self.af_vertical_swing = 0
 
-    def vertical_direction(self, newDirection):
-        if not isinstance(newDirection, int):
-            raise Exception("Wrong usage of method")
-        if newDirection > 0 and newDirection < 8:
-            self.af_vertical_direction = newDirection
-        else:
-            raise Exception("Direction out of range 1 - 7!")
-
     # Horizontal
     def horizontal_swing_on(self):
         self.af_vertical_swing = 1
@@ -160,14 +185,6 @@ class SplitAC:
     def horizontal_swing_off(self):
         self.af_vertical_swing = 0
 
-    def horizontal_direction(self, newDirection):
-        if not isinstance(newDirection, int):
-            raise Exception("Wrong usage of method")
-        if newDirection > 0 and newDirection < 8:
-            self.af_horizontal_direction = newDirection
-        else:
-            raise Exception("Direction out of range 1 - 7!")
-
     # Temperature setting
     def changeTemperature(self, newTemperature):
         # set temperature for degree C
@@ -227,11 +244,67 @@ class SplitAC:
         else:
             raise Exception("Wrong usage of the method!")
 
+    @property  # property to get display temperature in degree C
+    def display_temperature_degree(self) -> float | None:
+        data = None
+        if self._display_temperature is not None:
+            data = round(((self._display_temperature["value"] / 100 - 32) / 9 * 5), 1)
+        return data
+
+    @property  # property returns display temperature dict in 10 times of degree C
+    def display_temperature(self) -> dict[str, Any] | None:
+        return self._display_temperature
+
+    @display_temperature.setter
+    def display_temperature(self, properties: Any):
+        if isinstance(properties, (list, tuple)):
+            self._display_temperature = self._get_prop_from_json(
+                "display_temperature", properties
+            )
+        elif isinstance(properties, int) or isinstance(properties, float):
+            self._api._set_device_property(self.display_temperature["key"], properties)
+            self.refresh_properties()
+        else:
+            raise Exception("Wrong usage of the method!")
+
+    @property  # property to get outdoor temperature in degree C
+    def outdoor_temperature_degree(self) -> float | None:
+        data = None
+        if self._outdoor_temperature is not None:
+            data = round(((self._outdoor_temperature["value"] / 100 - 32) / 9 * 5), 1)
+        return data
+
+    @property  # property returns outdoor temperature dict in 10 times of degree C
+    def outdoor_temperature(self) -> dict[str, Any] | None:
+        return self._outdoor_temperature
+
+    @outdoor_temperature.setter
+    def outdoor_temperature(self, properties: Any):
+        if isinstance(properties, (list, tuple)):
+            self._outdoor_temperature = self._get_prop_from_json(
+                "outdoor_temperature", properties
+            )
+        elif isinstance(properties, int) or isinstance(properties, float):
+            self._api._set_device_property(self.outdoor_temperature["key"], properties)
+            self.refresh_properties()
+        else:
+            raise Exception("Wrong usage of the method!")
+
     @property  # property to get temperature in degree C
     def adjust_temperature_degree(self) -> float | None:
         data = None
         if self._adjust_temperature is not None:
-            data = round((self._adjust_temperature["value"] / 10), 1)
+            adjustTemperatureValue = self._adjust_temperature["value"]
+            if adjustTemperatureValue == 65535:
+                datapoints = self._get_device_property_history(
+                    self._adjust_temperature["key"]
+                )
+                # Get the latest setting other than invalid value
+                for datapoint in reversed(datapoints):
+                    if datapoint["datapoint"]["value"] != 65535:
+                        adjustTemperatureValue = int(datapoint["datapoint"]["value"])
+                        break
+            data = round((adjustTemperatureValue / 10), 1)
         return data
 
     @property  # property returns temperature dict in 10 times of degree C
@@ -294,6 +367,20 @@ class SplitAC:
         else:
             raise Exception("Wrong usage of the method!")
 
+    @property
+    def min_heat(self):
+        return self._min_heat
+
+    @min_heat.setter
+    def min_heat(self, properties):
+        if isinstance(properties, (list, tuple)):
+            self._min_heat = self._get_prop_from_json("min_heat", properties)
+        elif isinstance(properties, int):
+            self._api._set_device_property(self.min_heat["key"], properties)
+            self.refresh_properties()
+        else:
+            raise Exception("Wrong usage of the method!")
+
     @property
     def economy_mode(self):
         return self._economy_mode
@@ -332,7 +419,7 @@ class SplitAC:
         return self._af_horizontal_swing
 
     @af_horizontal_swing.setter
-    def af_horizontal_swing(self, properties):
+    def af_horizontal_swing(self, properties: Any):
         if isinstance(properties, (list, tuple)):
             self._af_horizontal_swing = self._get_prop_from_json(
                 "af_horizontal_swing", properties
@@ -348,16 +435,16 @@ class SplitAC:
         return self._af_vertical_direction
 
     @af_vertical_direction.setter
-    def af_vertical_direction(self, properties):
+    def af_vertical_direction(self, properties: Any) -> None:
         if isinstance(properties, (list, tuple)):
             self._af_vertical_direction = self._get_prop_from_json(
-                "af_vertical_move_step1", properties
+                "af_vertical_direction", properties
             )
         elif isinstance(properties, int):
             self._api._set_device_property(
                 self.af_vertical_direction["key"], properties
             )
-            # self.vertical_swing_off() ##If direction set then swing will be turned OFF
+            self.vertical_swing_off()  ##If direction set then swing will be turned OFF
             self.refresh_properties()
         else:
             raise Exception("Wrong usage of the method or direction out of range!")
@@ -383,7 +470,7 @@ class SplitAC:
         return self._device_name
 
     @device_name.setter
-    def device_name(self, properties):
+    def device_name(self, properties: Any) -> None:
         self._device_name = self._get_prop_from_json("device_name", properties)
 
     @property
@@ -402,7 +489,7 @@ class SplitAC:
         return data
 
     # Get a property history
-    def _get_device_property_history(self, propertyCode):
+    def _get_device_property_history(self, propertyCode: int) -> Any:
         propertyHistory = self._api._get_device_property(propertyCode)
         propertyHistory = propertyHistory.json()
 
@@ -410,19 +497,18 @@ class SplitAC:
 
     # Translate the operation mode to descriptive values and reverse
     def _operation_mode_translate(self, operation_mode: Any) -> Any:
-
         DICT_OPERATION_MODE = {
             "off": 0,
-            "unknown": 1,
-            "auto": 2,
-            "cool": 3,
+            "heat": 1,
+            "cool": 2,
+            "auto": 3,
             "dry": 4,
             "fan_only": 5,
-            "heat": 6,
+            "heat_alt": 6,
             0: "off",
-            1: "unknown",
-            2: "auto",
-            3: "cool",
+            1: "heat",
+            2: "cool",
+            3: "auto",
             4: "dry",
             5: "fan_only",
             6: "heat",
